@@ -41,6 +41,7 @@
 #include <GraphMol/ROMol.h>
 #include <GraphMol/new_canon.h>
 #include <GraphMol/FileParsers/MolSGroupParsing.h>
+#include "Subset.h"
 
 const int ci_LOCAL_INF = static_cast<int>(1e8);
 
@@ -700,6 +701,7 @@ std::vector<std::unique_ptr<ROMol>> getTheFrags(
   }
   int nFrags = getMolFrags(mol, *frags);
   std::vector<std::unique_ptr<RWMol>> res;
+
   if (nFrags == 1) {
     res.emplace_back(new RWMol(mol));
     if (fragsMolAtomMapping) {
@@ -777,35 +779,14 @@ std::vector<std::unique_ptr<ROMol>> getTheFrags(
         // empirical. This is mainly intended to catch situations like proteins
         // where you have a bunch of single-atom fragments (waters); the
         // standard approach below ends up being horribly inefficient there
-        res.emplace_back(new RWMol());
-        auto &frag = res.back();
-        std::map<unsigned int, unsigned int> atomIdxMap;
-        for (auto aid : comp) {
-          atomIdxMap[aid] =
-              frag->addAtom(mol.getAtomWithIdx(aid)->copy(), false, true);
-        }
-        for (auto bond : mol.bonds()) {
-          if (atomsInFrag[bond->getBeginAtomIdx()] &&
-              atomsInFrag[bond->getEndAtomIdx()]) {
-            auto bondCopy = bond->copy();
-            bondCopy->setBeginAtomIdx(atomIdxMap[bond->getBeginAtomIdx()]);
-            bondCopy->setEndAtomIdx(atomIdxMap[bond->getEndAtomIdx()]);
-            frag->addBond(bondCopy, true);
-          }
-        }
-        if (copyConformers) {
-          for (auto cit = mol.beginConformers(); cit != mol.endConformers();
-               ++cit) {
-            auto *conf = new Conformer(frag->getNumAtoms());
-            conf->setId((*cit)->getId());
-            conf->set3D((*cit)->is3D());
-            unsigned int cidx = 0;
-            for (auto ai : comp) {
-              conf->setAtomPos(cidx++, (*cit)->getAtomPos(ai));
-            }
-            frag->addConformer(conf);
-          }
-        }
+        SubsetOptions opts{.sanitize = sanitizeFrags,
+                           .clearComputedProps = true,
+                           .copyCoordinates = copyConformers,
+                           .method = SubsetMethod::BONDS_BETWEEN_ATOMS};
+        std::vector<unsigned int> atoms{comp.begin(), comp.end()};
+        SubsetInfo info;
+        auto submol = copyMolSubset(mol, atoms, info, opts);
+        res.push_back(std::move(submol));
       } else {
         res.emplace_back(new RWMol(mol));
         auto &frag = res.back();
@@ -842,6 +823,7 @@ std::vector<std::unique_ptr<ROMol>> getTheFrags(
   }
   return finalRes;
 }
+
 }  // namespace
 std::vector<ROMOL_SPTR> getMolFrags(const ROMol &mol, bool sanitizeFrags,
                                     INT_VECT *frags,
