@@ -98,8 +98,8 @@ void processCXSmilesLabels(RWMol &mol) {
                       symb.substr(0, symb.size() - 2));
         atom->clearProp(common_properties::atomLabel);
       }
-    } else if (atom->getAtomicNum() == 0 && !atom->hasQuery() && !atom->getIsotope() &&
-               atom->getSymbol() == "*") {
+    } else if (atom->getAtomicNum() == 0 && !atom->hasQuery() &&
+               !atom->getIsotope() && atom->getSymbol() == "*") {
       addquery(makeAAtomQuery(), "", mol, atom->getIdx());
     }
   }
@@ -2023,9 +2023,11 @@ std::string get_coords_block(const ROMol &mol,
 
 std::string get_atom_props_block(const ROMol &mol,
                                  const std::vector<unsigned int> &atomOrder) {
-  std::vector<std::string> skip = {common_properties::atomLabel,
-                                   common_properties::molFileValue,
-                                   common_properties::molParity};
+  static const std::array<std::string_view, 7> skip = {
+      common_properties::atomLabel,       common_properties::molFileValue,
+      common_properties::molParity,       common_properties::molAtomMapNumber,
+      common_properties::molStereoCare,   common_properties::molRxnExactChange,
+      common_properties::molInversionFlag};
   std::string res = "";
   unsigned int which = 0;
   for (auto idx : atomOrder) {
@@ -2037,7 +2039,7 @@ std::string get_atom_props_block(const ROMol &mol,
       if (std::find(skip.begin(), skip.end(), pn) == skip.end()) {
         std::string pv = atom->getProp<std::string>(pn);
         if (pn == "dummyLabel" &&
-            (isAttachmentPoint ||
+            (isAttachmentPoint || pv == "*" ||
              std::find(SmilesParseOps::pseudoatoms.begin(),
                        SmilesParseOps::pseudoatoms.end(),
                        pv) != SmilesParseOps::pseudoatoms.end())) {
