@@ -3,7 +3,10 @@
 #include <stdexcept>
 
 #include <GraphMol/RWMol.h>
+#include <Geometry/Transform3D.h>
+#include <GraphMol/MolTransforms/MolTransforms.h>
 #include <GraphMol/SmilesParse/SmilesParse.h>
+#include <GraphMol/SmilesParse/SmilesWrite.h>
 #include <GraphMol/Substruct/SubstructMatch.h>
 
 #include <RDGeneral/BoostStartInclude.h>
@@ -371,14 +374,17 @@ void extractFeatureCoords(
   }
 }
 
-void extractCustomFeatureCoords(const unsigned int nSelectedAtoms,
-                                const ShapeInputOptions &shapeOpts,
-                                const RDGeom::Point3D &ave,
-                                unsigned int &numFeatures, ShapeInput &res,
-                                std::vector<double> &rad_vector) {
+void extractCustomFeatureCoords(
+    const unsigned int nSelectedAtoms, const ShapeInputOptions &shapeOpts,
+    const RDGeom::Point3D &ave, unsigned int &numFeatures, ShapeInput &res,
+    std::vector<double> &rad_vector,
+    const std::unique_ptr<RDGeom::Transform3D> &trans) {
   for (const auto &feature : shapeOpts.customFeatures) {
     unsigned int feature_type = std::get<0>(feature);
     RDGeom::Point3D floc = std::get<1>(feature);
+    if (trans) {
+      trans->TransformPoint(floc);
+    }
     double radius = std::get<2>(feature);
     floc -= ave;
     DEBUG_MSG("custom feature type " << feature_type << " (" << floc << ")");
@@ -394,7 +400,8 @@ void extractCustomFeatureCoords(const unsigned int nSelectedAtoms,
 }
 
 }  // namespace
-// The conformer is left where it is, the shape is translated to the origin.
+// The conformer is left where it is, the shape is translated to the origin
+// and aligned to the principal axes.
 ShapeInput PrepareConformer(const ROMol &mol, int confId,
                             const ShapeInputOptions &shapeOpts) {
   Align3D::setUseCutOff(true);
@@ -420,24 +427,61 @@ ShapeInput PrepareConformer(const ROMol &mol, int confId,
   std::vector<double> rad_vector(nAlignmentAtoms);
   res.atom_type_vector.resize(nAlignmentAtoms, 0);
 
+  Conformer confCp(conformer);
+  std::unique_ptr<RDGeom::Transform3D> trans;
+  RDGeom::Point3D initialCentroid;
+  if (shapeOpts.normalize) {
+    // This ignores hydrogens by default, which is what is needed here.
+    trans.reset(MolTransforms::computeCanonicalTransform(confCp));
+    MolTransforms::transformConformer(confCp, *trans);
+
+    // Save the rotation part of the transformation.  The coords will be centred
+    // below and left that way so we don't need to keep the translation.
+    for (unsigned int i = 0, k = 0; i < 3; ++i) {
+      for (unsigned int j = 0; j < 3; ++j, ++k) {
+        res.inertialRot[k] = trans->getValUnchecked(i, j);
+      }
+    }
+
+    // Work back from the canonical transformation to get the initial
+    // centroid by the inverse rotation of the current translation
+    // but not taking its negative so it can be used directly as the shift.
+    initialCentroid = RDGeom::Point3D{trans->getValUnchecked(0, 3),
+                                      trans->getValUnchecked(1, 3),
+                                      trans->getValUnchecked(2, 3)};
+    RDGeom::Transform3D invTrans;
+    invTrans.setValUnchecked(0, 0, trans->getValUnchecked(0, 0));
+    invTrans.setValUnchecked(0, 1, trans->getValUnchecked(1, 0));
+    invTrans.setValUnchecked(0, 2, trans->getValUnchecked(2, 0));
+    invTrans.setValUnchecked(1, 0, trans->getValUnchecked(0, 1));
+    invTrans.setValUnchecked(1, 1, trans->getValUnchecked(1, 1));
+    invTrans.setValUnchecked(1, 2, trans->getValUnchecked(2, 1));
+    invTrans.setValUnchecked(2, 0, trans->getValUnchecked(0, 2));
+    invTrans.setValUnchecked(2, 1, trans->getValUnchecked(1, 2));
+    invTrans.setValUnchecked(2, 2, trans->getValUnchecked(2, 2));
+    invTrans.TransformPoint(initialCentroid);
+  }
+  RWMol molCp(mol);
+  molCp.removeConformer(0);
+  molCp.addConformer(new Conformer(confCp), true);
+
   RDGeom::Point3D ave;
   unsigned int nSelectedAtoms = 0;
-  extractAtomRadii(conformer, nAtoms, shapeOpts, ave, nSelectedAtoms,
-                   rad_vector);
-
+  // Calculates the average coord as by-product
+  extractAtomRadii(confCp, nAtoms, shapeOpts, ave, nSelectedAtoms, rad_vector);
   // translate steric center to origin
   DEBUG_MSG("steric center: (" << ave << ")");
   res.shift = {-ave.x, -ave.y, -ave.z};
   res.coord.resize(nAlignmentAtoms * 3);
 
-  extractAtomCoords(conformer, nAtoms, shapeOpts, ave, res.coord);
+  extractAtomCoords(confCp, nAtoms, shapeOpts, ave, res.coord);
   unsigned int numFeatures = 0;
   if (shapeOpts.customFeatures.empty()) {
-    extractFeatureCoords(conformer, nAtoms, nSelectedAtoms, feature_idx_type,
+    extractFeatureCoords(confCp, nAtoms, nSelectedAtoms, feature_idx_type,
                          shapeOpts, ave, numFeatures, res, rad_vector);
   } else if (shapeOpts.useColors) {
     extractCustomFeatureCoords(nSelectedAtoms, shapeOpts, ave, numFeatures, res,
-                               rad_vector);
+                               rad_vector, trans);
   }
 
   // Now cut the final vectors down to the actual number of atoms and
@@ -467,6 +511,9 @@ ShapeInput PrepareConformer(const ROMol &mol, int confId,
         res.coord.data(), res.alpha_vector, res.colorAtomType2IndexVectorMap);
     DEBUG_MSG("sof: " << res.sof);
   }
+
+  // Finally set the shift to the negative of the original average coords.
+  res.shift = {initialCentroid.x, initialCentroid.y, initialCentroid.z};
   return res;
 }
 
@@ -499,7 +546,6 @@ std::pair<double, double> AlignShape(const ShapeInput &refShape,
       fitShape.volumeAtomIndexVector, fitMapCp, fitShape.sov, fitShape.sof,
       !jointColorAtomTypeSet.empty(), max_preiters, max_postiters, opt_param,
       matrix.data(), nbr_st, nbr_ct);
-
   DEBUG_MSG("Done!");
   DEBUG_MSG("nbr_st: " << nbr_st);
   DEBUG_MSG("nbr_ct: " << nbr_ct);
@@ -512,36 +558,73 @@ std::pair<double, double> AlignShape(const ShapeInput &refShape,
 }
 
 void TransformConformer(const std::vector<double> &finalTrans,
-                        const std::vector<float> &matrix, ShapeInput &fitShape,
-                        Conformer &fitConf) {
-  // we reuse/modify the coord member of fitShape in order to avoid memory
-  // allocations
-  const unsigned int nAtoms = fitConf.getOwningMol().getNumAtoms();
-  if (nAtoms > fitShape.volumeAtomIndexVector.size()) {
-    // Hs are missing... make sure we have space for them
-    fitShape.coord.resize(3 * nAtoms);
-  }
-  // initialize the fitShape coords with the starting atomic positions from
-  // the conformer shifted to the center of "mass" coordinates.
-  for (unsigned int i = 0; i < nAtoms; ++i) {
-    const auto &pos = fitConf.getAtomPos(i);
-    fitShape.coord[i * 3] = pos.x + fitShape.shift[0];
-    fitShape.coord[i * 3 + 1] = pos.y + fitShape.shift[1];
-    fitShape.coord[i * 3 + 2] = pos.z + fitShape.shift[2];
-  }
+                        const std::vector<double> &finalRot,
+                        const std::vector<float> &matrix,
+                        const ShapeInput &fitShape, Conformer &fitConf) {
+  // Move fitConf to fitShape's initial centroid and principal axes
+  RDGeom::Transform3D transform0;
+  transform0.SetTranslation(
+      RDGeom::Point3D{fitShape.shift[0], fitShape.shift[1], fitShape.shift[2]});
 
-  std::vector<float> transformed(nAtoms * 3);
-  Align3D::VApplyRotTransMatrix(transformed.data(), fitShape.coord.data(),
-                                nAtoms, matrix.data());
+  RDGeom::Transform3D transform1;
+  transform1.setValUnchecked(0, 0, fitShape.inertialRot[0]);
+  transform1.setValUnchecked(0, 1, fitShape.inertialRot[1]);
+  transform1.setValUnchecked(0, 2, fitShape.inertialRot[2]);
+  transform1.setValUnchecked(1, 0, fitShape.inertialRot[3]);
+  transform1.setValUnchecked(1, 1, fitShape.inertialRot[4]);
+  transform1.setValUnchecked(1, 2, fitShape.inertialRot[5]);
+  transform1.setValUnchecked(2, 0, fitShape.inertialRot[6]);
+  transform1.setValUnchecked(2, 1, fitShape.inertialRot[7]);
+  transform1.setValUnchecked(2, 2, fitShape.inertialRot[8]);
+  transform1.setValUnchecked(2, 3, 0.0);
+  transform1.setValUnchecked(3, 0, 0.0);
+  transform1.setValUnchecked(3, 1, 0.0);
+  transform1.setValUnchecked(3, 2, 0.0);
+  transform1.setValUnchecked(3, 3, 1.0);
 
-  // now set the coordinates in the conformer; undo the shift of the reference
-  // shape to center of "mass" coordinates
-  for (unsigned i = 0; i < nAtoms; ++i) {
-    RDGeom::Point3D &pos = fitConf.getAtomPos(i);
-    pos.x = transformed[i * 3] - finalTrans[0];
-    pos.y = transformed[(i * 3) + 1] - finalTrans[1];
-    pos.z = transformed[(i * 3) + 2] - finalTrans[2];
-  }
+  // Apply the overlay matrix.
+  // The matrix comes in as a linear form of a 3x3 rotation matrix and a
+  // 3x1 translation.  Convert that into a RDGeom::Trans3D
+  RDGeom::Transform3D transform2;
+  transform2.setValUnchecked(0, 0, matrix[0]);
+  transform2.setValUnchecked(0, 1, matrix[1]);
+  transform2.setValUnchecked(0, 2, matrix[2]);
+  transform2.setValUnchecked(0, 3, matrix[9]);
+  transform2.setValUnchecked(1, 0, matrix[3]);
+  transform2.setValUnchecked(1, 1, matrix[4]);
+  transform2.setValUnchecked(1, 2, matrix[5]);
+  transform2.setValUnchecked(1, 3, matrix[10]);
+  transform2.setValUnchecked(2, 0, matrix[6]);
+  transform2.setValUnchecked(2, 1, matrix[7]);
+  transform2.setValUnchecked(2, 2, matrix[8]);
+  transform2.setValUnchecked(2, 3, matrix[11]);
+  transform2.setValUnchecked(3, 0, 0.0);
+  transform2.setValUnchecked(3, 1, 0.0);
+  transform2.setValUnchecked(3, 2, 0.0);
+  transform2.setValUnchecked(3, 3, 1.0);
+
+  // Now apply the final rotation and final translation, which
+  // is to put it into the reference shape's frame of reference.
+  RDGeom::Transform3D transform3;
+  transform3.setValUnchecked(0, 0, finalRot[0]);
+  transform3.setValUnchecked(0, 1, finalRot[1]);
+  transform3.setValUnchecked(0, 2, finalRot[2]);
+  transform3.setValUnchecked(0, 3, finalTrans[0]);
+  transform3.setValUnchecked(1, 0, finalRot[3]);
+  transform3.setValUnchecked(1, 1, finalRot[4]);
+  transform3.setValUnchecked(1, 2, finalRot[5]);
+  transform3.setValUnchecked(1, 3, finalTrans[1]);
+  transform3.setValUnchecked(2, 0, finalRot[6]);
+  transform3.setValUnchecked(2, 1, finalRot[7]);
+  transform3.setValUnchecked(2, 2, finalRot[8]);
+  transform3.setValUnchecked(2, 3, finalTrans[2]);
+  transform3.setValUnchecked(3, 0, 0.0);
+  transform3.setValUnchecked(3, 1, 0.0);
+  transform3.setValUnchecked(3, 2, 0.0);
+  transform3.setValUnchecked(3, 3, 1.0);
+
+  auto finalTransform = transform3 * transform2 * transform1 * transform0;
+  MolTransforms::transformConformer(fitConf, finalTransform);
 }
 
 std::pair<double, double> AlignMolecule(
@@ -555,14 +638,23 @@ std::pair<double, double> AlignMolecule(
   auto fitShape = PrepareConformer(fit, fitConfId, shapeOpts);
   auto tanis = AlignShape(refShape, fitShape, matrix, opt_param, max_preiters,
                           max_postiters);
-
   // transform fit coords
   Conformer &fit_conformer = fit.getConformer(fitConfId);
   std::vector<double> finalTrans{0.0, 0.0, 0.0};
+  std::vector<double> finalRot{1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0};
   if (applyRefShift) {
-    finalTrans = refShape.shift;
+    finalTrans = std::vector<double>{-refShape.shift[0], -refShape.shift[1],
+                                     -refShape.shift[2]};
+    // set the final rotation to the inverse of the original inertialRot
+    // of the reference.
+    finalRot =
+        std::vector<double>{refShape.inertialRot[0], refShape.inertialRot[3],
+                            refShape.inertialRot[6], refShape.inertialRot[1],
+                            refShape.inertialRot[4], refShape.inertialRot[7],
+                            refShape.inertialRot[2], refShape.inertialRot[5],
+                            refShape.inertialRot[8]};
   }
-  TransformConformer(finalTrans, matrix, fitShape, fit_conformer);
+  TransformConformer(finalTrans, finalRot, matrix, fitShape, fit_conformer);
   fit.setProp("shape_align_shape_tanimoto", tanis.first);
   fit.setProp("shape_align_color_tanimoto", tanis.second);
 
@@ -582,11 +674,11 @@ std::pair<double, double> AlignMolecule(
 std::pair<double, double> AlignMolecule(
     const ROMol &ref, ROMol &fit, std::vector<float> &matrix,
     const ShapeInputOptions &refShapeOpts,
-    const ShapeInputOptions &probeShapeOpts, int refConfId, int fitConfId,
+    const ShapeInputOptions &fitShapeOpts, int refConfId, int fitConfId,
     double opt_param, unsigned int max_preiters, unsigned int max_postiters) {
   Align3D::setUseCutOff(true);
 
-  if (refShapeOpts.useColors != probeShapeOpts.useColors) {
+  if (refShapeOpts.useColors != fitShapeOpts.useColors) {
     BOOST_LOG(rdWarningLog)
         << "useColor values inconsistent between the reference and fit molecules. Colors will not be used in the alignment."
         << std::endl;
@@ -596,7 +688,7 @@ std::pair<double, double> AlignMolecule(
   auto refShape = PrepareConformer(ref, refConfId, refShapeOpts);
   bool applyRefShift = true;
   auto scores =
-      AlignMolecule(refShape, fit, matrix, probeShapeOpts, fitConfId, opt_param,
+      AlignMolecule(refShape, fit, matrix, fitShapeOpts, fitConfId, opt_param,
                     max_preiters, max_postiters, applyRefShift);
   return scores;
 }
